
/*****************************************************************
-- Nom:                 ALTACLIENT
--
-- Descripci:          Dna d'alta un client
--
-- Parmetres entrada:
						pIDCONCESSIONARI CLIENT.IDCONCESSIONARI%TYPE,
						pNIF CLIENT.NIF%TYPE,
						pNOM CLIENT.NOM%TYPE,
						pCOGNOM1 CLIENT.COGNOM1%TYPE,
						pCOGNOM2 CLIENT.COGNOM2%TYPE,
						pADRECA CLIENT.ADRECA%TYPE,
						pPOBLACIO CLIENT.POBLACIO%TYPE,
						pCODIPOSTAL CLIENT.CODIPOSTAL%TYPE,
						pTELEFON CLIENT.TELEFON%TYPE,
						pMAIL CLIENT.MAIL%TYPE
--
-- Error controlats:   - Client duplicat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ALTACLIENT" ( 
					   pIDCONCESSIONARI CLIENT.IDCONCESSIONARI%TYPE,
					   pNIF CLIENT.NIF%TYPE,
					   pNOM CLIENT.NOM%TYPE,
					   pCOGNOM1 CLIENT.COGNOM1%TYPE,
					   pCOGNOM2 CLIENT.COGNOM2%TYPE,
					   pADRECA CLIENT.ADRECA%TYPE,
					   pPOBLACIO CLIENT.POBLACIO%TYPE,
					   pCODIPOSTAL CLIENT.CODIPOSTAL%TYPE,
					   pTELEFON CLIENT.TELEFON%TYPE,
					   pMAIL CLIENT.MAIL%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCONCESSIONARI||';'||pNIF||';'||pNOM||';'||pCOGNOM1||';'||pCOGNOM2||';'||pADRECA||';'||pPOBLACIO||';'||
			pCODIPOSTAL||';'||pTELEFON||';'||pMAIL;

	INSERT INTO CLIENT (IDCONCESSIONARI,NIF,NOM,COGNOM1,COGNOM2,ADRECA,POBLACIO,CODIPOSTAL,TELEFON,MAIL,DATAREGISTRE) 
	VALUES (pIDCONCESSIONARI,pNIF,pNOM,pCOGNOM1,pCOGNOM2,pADRECA,pPOBLACIO,pCODIPOSTAL,pTELEFON,pMAIL,SYSDATE);
    
	INSERIRLOG ('ALTACLIENT',PARAMS,RESPOSTA);
	
    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'El client ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
          INSERIRLOG ('ALTACLIENT',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en alta client: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
		  INSERIRLOG ('ALTACLIENT',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARCLIENT
--
-- Descripci:          Modifica les dades dun client
--
-- Parmetres entrada:
						pIDCLIENT CLIENT.IDCLIENT%TYPE
						pIDCONCESSIONARI CLIENT.IDCONCESSIONARI%TYPE,
						pNIF CLIENT.NIF%TYPE,
						pNOM CLIENT.NOM%TYPE,
						pCOGNOM1 CLIENT.COGNOM1%TYPE,
						pCOGNOM2 CLIENT.COGNOM2%TYPE,
						pADRECA CLIENT.ADRECA%TYPE,
						pPOBLACIO CLIENT.POBLACIO%TYPE,
						pCODIPOSTAL CLIENT.CODIPOSTAL%TYPE,
						pTELEFON CLIENT.TELEFON%TYPE,
						pMAIL CLIENT.MAIL%TYPE,
						pDATAREGISTRE CLIENT.DATAREGISTRE%TYPE,
						pDATABAIXA CLIENT.DATABAIXA%TYPE
--
-- Error controlats:   - Client no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARCLIENT" ( 
					   pIDCLIENT CLIENT.IDCLIENT%TYPE,
					   pIDCONCESSIONARI CLIENT.IDCONCESSIONARI%TYPE,
					   pNIF CLIENT.NIF%TYPE,
					   pNOM CLIENT.NOM%TYPE,
					   pCOGNOM1 CLIENT.COGNOM1%TYPE,
					   pCOGNOM2 CLIENT.COGNOM2%TYPE,
					   pADRECA CLIENT.ADRECA%TYPE,
					   pPOBLACIO CLIENT.POBLACIO%TYPE,
					   pCODIPOSTAL CLIENT.CODIPOSTAL%TYPE,
					   pTELEFON CLIENT.TELEFON%TYPE,
					   pMAIL CLIENT.MAIL%TYPE,
					   pDATAREGISTRE CLIENT.DATAREGISTRE%TYPE,					   
					   pDATABAIXA CLIENT.DATABAIXA%TYPE,		
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCLIENT||';'||pIDCONCESSIONARI||';'||pNIF||';'||pNOM||';'||pCOGNOM1||';'||pCOGNOM2||';'||pADRECA||';'||pPOBLACIO||';'||
			pCODIPOSTAL||';'||pTELEFON||';'||pMAIL||';'||pDATAREGISTRE||';'||pDATABAIXA;

	SELECT COUNT(*) INTO COUNT_TMP FROM CLIENT WHERE IDCLIENT = pIDCLIENT;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
    UPDATE CLIENT
    SET IDCONCESSIONARI = pIDCONCESSIONARI, 
		NIF = pNIF, 
		NOM = pNOM, 
		COGNOM1 = pCOGNOM1, 
		COGNOM2 = pCOGNOM2, 
		ADRECA = pADRECA, 
		POBLACIO = pPOBLACIO, 
		CODIPOSTAL = pCODIPOSTAL, 
		TELEFON = pTELEFON, 
		MAIL = pMAIL,
		DATAREGISTRE = pDATAREGISTRE,
		DATABAIXA = pDATABAIXA
	WHERE IDCLIENT = pIDCLIENT;		

	INSERIRLOG ('MODIFICARCLIENT',PARAMS,RESPOSTA);
	
	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap client amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARCLIENT',PARAMS,RESPOSTA);
		   
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del client : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  INSERIRLOG ('MODIFICARCLIENT',PARAMS,RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 ESBORRARCLIENT
--
-- Descripci:          Esborra el client de la bd
--
-- Parmetres entrada:
						pIDCLIENT CLIENT.IDCLIENT%TYPE
--
-- Error controlats:   - Client no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARCLIENT" ( 
					   pIDCLIENT CLIENT.IDCLIENT%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCLIENT;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM CLIENT WHERE IDCLIENT = pIDCLIENT;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM CLIENT WHERE IDCLIENT = pIDCLIENT;

	INSERIRLOG ('ESBORRARCLIENT',PARAMS,RESPOSTA);
		
	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap client amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARCLIENT',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant client : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('ESBORRARCLIENT',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ALTAVEHICLECLIENT
--
-- Descripci:          Dna dalta un vehicle dun client
--
-- Parmetres entrada:
						pBASTIDOR VEHICLECLIENT.BASTIDOR%TYPE,
						pIDCLIENT VEHICLECLIENT.IDCLIENT%TYPE,
						pMATRICULA VEHICLECLIENT.MATRICULA%TYPE,
						pANYMODEL VEHICLECLIENT.ANYMODEL%TYPE,
						pIDCOLOR VEHICLECLIENT.IDCOLOR%TYPE,
						pIDCONFIGURACIO VEHICLECLIENT.IDCONFIGURACIO%TYPE,
						pTOTALKM VEHICLECLIENT.TOTALKM%TYPE,
						pDATAMATRICULACIO VEHICLECLIENT.DATAMATRICULACIO%TYPE,
						pDATAULTIMAREVISIO VEHICLECLIENT.DATAULTIMAREVISIO%TYPE,
						pDATAREGISTRE VEHICLECLIENT.DATAREGISTRE%TYPE,
						pDATABAIXA VEHICLECLIENT.DATABAIXA%TYPE
--
-- Error controlats:   - El client no existeix 
					   - Vehicle duplicat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ALTAVEHICLECLIENT" ( 
					   pBASTIDOR VEHICLECLIENT.BASTIDOR%TYPE,
					   pIDCLIENT VEHICLECLIENT.IDCLIENT%TYPE,
					   pMATRICULA VEHICLECLIENT.MATRICULA%TYPE,
					   pANYMODEL VEHICLECLIENT.ANYMODEL%TYPE,
					   pIDCOLOR VEHICLECLIENT.IDCOLOR%TYPE,
					   pIDCONFIGURACIO VEHICLECLIENT.IDCONFIGURACIO%TYPE,
					   pTOTALKM VEHICLECLIENT.TOTALKM%TYPE,
					   pDATAMATRICULACIO VEHICLECLIENT.DATAMATRICULACIO%TYPE,
					   pDATAULTIMAREVISIO VEHICLECLIENT.DATAULTIMAREVISIO%TYPE,
					   pDATAREGISTRE VEHICLECLIENT.DATAREGISTRE%TYPE,
					   pDATABAIXA VEHICLECLIENT.DATABAIXA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pBASTIDOR||';'||pIDCLIENT||';'||pMATRICULA||';'||pANYMODEL||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||pTOTALKM||';'||
			pDATAMATRICULACIO||';'||pDATAULTIMAREVISIO||';'||pDATAREGISTRE||';'||pDATABAIXA;

	-- Validem que existeix el client
	SELECT COUNT(*) INTO COUNT_TMP FROM CLIENT WHERE IDCLIENT = pIDCLIENT AND DATABAIXA IS NULL;
	IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;

	INSERT INTO VEHICLECLIENT (BASTIDOR,IDCLIENT,MATRICULA,ANYMODEL,IDCOLOR,IDCONFIGURACIO,TOTALKM,DATAMATRICULACIO,DATAULTIMAREVISIO,DATAREGISTRE,DATABAIXA) 
	VALUES (pBASTIDOR,pIDCLIENT,pMATRICULA,pANYMODEL,pIDCOLOR,pIDCONFIGURACIO,pTOTALKM,pDATAMATRICULACIO,pDATAULTIMAREVISIO,SYSDATE,pDATABAIXA); 
	
	INSERIRLOG ('ALTAVEHICLECLIENT',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap client amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ALTAVEHICLECLIENT',PARAMS,RESPOSTA);
		
		WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'El vehicle ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ALTAVEHICLECLIENT',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error en alta vehicle client: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ALTAVEHICLECLIENT',PARAMS,RESPOSTA);		  
    
END;
/

/*****************************************************************
-- Nom:                 MODIFICARVEHICLECLIENT
--
-- Descripci:          Dna dalta un vehicle dun client
--
-- Parmetres entrada:
						pIDVEHICLECLIENT VEHICLECLIENT.IDVEHICLECLIENT%TYPE,
						pBASTIDOR VEHICLECLIENT.BASTIDOR%TYPE,
						pIDCLIENT VEHICLECLIENT.IDCLIENT%TYPE,
						pMATRICULA VEHICLECLIENT.MATRICULA%TYPE,
						pANYMODEL VEHICLECLIENT.ANYMODEL%TYPE,
						pIDCOLOR VEHICLECLIENT.IDCOLOR%TYPE,
						pIDCONFIGURACIO VEHICLECLIENT.IDCONFIGURACIO%TYPE,
						pTOTALKM VEHICLECLIENT.TOTALKM%TYPE,
						pDATAMATRICULACIO VEHICLECLIENT.DATAMATRICULACIO%TYPE,
						pDATAULTIMAREVISIO VEHICLECLIENT.DATAULTIMAREVISIO%TYPE,
						pDATAREGISTRE VEHICLECLIENT.DATAREGISTRE%TYPE,
						pDATABAIXA VEHICLECLIENT.DATABAIXA%TYPE						
--
-- Error controlats:   - Vehicle no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARVEHICLECLIENT" ( 
					   pIDVEHICLECLIENT VEHICLECLIENT.IDVEHICLECLIENT%TYPE,
					   pBASTIDOR VEHICLECLIENT.BASTIDOR%TYPE,
					   pIDCLIENT VEHICLECLIENT.IDCLIENT%TYPE,
					   pMATRICULA VEHICLECLIENT.MATRICULA%TYPE,
					   pANYMODEL VEHICLECLIENT.ANYMODEL%TYPE,
					   pIDCOLOR VEHICLECLIENT.IDCOLOR%TYPE,
					   pIDCONFIGURACIO VEHICLECLIENT.IDCONFIGURACIO%TYPE,
					   pTOTALKM VEHICLECLIENT.TOTALKM%TYPE,
					   pDATAMATRICULACIO VEHICLECLIENT.DATAMATRICULACIO%TYPE,
					   pDATAULTIMAREVISIO VEHICLECLIENT.DATAULTIMAREVISIO%TYPE,
					   pDATAREGISTRE VEHICLECLIENT.DATAREGISTRE%TYPE,
					   pDATABAIXA VEHICLECLIENT.DATABAIXA%TYPE,		
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDVEHICLECLIENT||';'||pBASTIDOR||';'||pIDCLIENT||';'||pMATRICULA||';'||pANYMODEL||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||pTOTALKM||';'||
			pDATAMATRICULACIO||';'||pDATAULTIMAREVISIO||';'||pDATAREGISTRE||';'||pDATABAIXA;

	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE IDVEHICLECLIENT = pIDVEHICLECLIENT;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
    UPDATE VEHICLECLIENT
    SET BASTIDOR = pBASTIDOR,
		IDCLIENT = pIDCLIENT, 
		MATRICULA = pMATRICULA, 
		ANYMODEL = pANYMODEL, 
		IDCOLOR = pIDCOLOR, 
		IDCONFIGURACIO = pIDCONFIGURACIO, 
		TOTALKM = pTOTALKM, 
		DATAMATRICULACIO = pDATAMATRICULACIO, 
		DATAULTIMAREVISIO = pDATAULTIMAREVISIO, 
		DATAREGISTRE = pDATAREGISTRE, 
		DATABAIXA = pDATABAIXA
	WHERE IDVEHICLECLIENT = pIDVEHICLECLIENT;

	INSERIRLOG ('MODIFICARVEHICLECLIENT',PARAMS,RESPOSTA);
	
	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap vehicle amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARVEHICLECLIENT',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del vehicle client : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARVEHICLECLIENT',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARVEHICLECLIENT
--
-- Descripci:          Esborra el vehicle d'un client
--
-- Parmetres entrada:
						pIDVEHICLECLIENT VEHICLECLIENT.IDVEHICLECLIENT%TYPE
--
-- Error controlats:   - Vehicle no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARVEHICLECLIENT" ( 
					   pIDVEHICLECLIENT VEHICLECLIENT.IDVEHICLECLIENT%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDVEHICLECLIENT;

	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE IDVEHICLECLIENT = pIDVEHICLECLIENT;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM VEHICLECLIENT WHERE IDVEHICLECLIENT = pIDVEHICLECLIENT;

	INSERIRLOG ('ESBORRARVEHICLECLIENT',PARAMS,RESPOSTA);
	
	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap vehicle amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARVEHICLECLIENT',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant vehicle client : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ESBORRARVEHICLECLIENT',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 AFEGIRVEHICLESTOCK
--
-- Descripci:          Dna d'alta un vehicle en stock per un concessionari
--
-- Parmetres entrada:
						pBASTIDOR VEHICLESTOCK.BASTIDOR%TYPE,
						pIDCONCESSIONARI VEHICLESTOCK.IDCONCESSIONARI%TYPE,
						pIDTIPUSVEHICLE VEHICLESTOCK.IDTIPUSVEHICLE%TYPE,
						pMATRICULA VEHICLESTOCK.MATRICULA%TYPE,
						pANYMODEL VEHICLESTOCK.ANYMODEL%TYPE,
						pIDCOLOR VEHICLESTOCK.IDCOLOR%TYPE,
						pIDCONFIGURACIO VEHICLESTOCK.IDCONFIGURACIO%TYPE,
						pTOTALKM VEHICLESTOCK.TOTALKM%TYPE,
						pDATAMATRICULACIO VEHICLESTOCK.DATAMATRICULACIO%TYPE,
						pPREU VEHICLESTOCK.PREU%TYPE,
						pDATAENTRADA VEHICLESTOCK.DATAENTRADA%TYPE,
						pDATASORTIDA VEHICLESTOCK.DATASORTIDA%TYPE
--
-- Error controlats:   - Vehicle duplicat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "AFEGIRVEHICLESTOCK" ( 
					   pBASTIDOR VEHICLESTOCK.BASTIDOR%TYPE,
					   pIDCONCESSIONARI VEHICLESTOCK.IDCONCESSIONARI%TYPE,
					   pIDTIPUSVEHICLE VEHICLESTOCK.IDTIPUSVEHICLE%TYPE,
					   pMATRICULA VEHICLESTOCK.MATRICULA%TYPE,
					   pANYMODEL VEHICLESTOCK.ANYMODEL%TYPE,
					   pIDCOLOR VEHICLESTOCK.IDCOLOR%TYPE,
					   pIDCONFIGURACIO VEHICLESTOCK.IDCONFIGURACIO%TYPE,
					   pTOTALKM VEHICLESTOCK.TOTALKM%TYPE,
					   pDATAMATRICULACIO VEHICLESTOCK.DATAMATRICULACIO%TYPE,
					   pPREU VEHICLESTOCK.PREU%TYPE,
					   pDATAENTRADA VEHICLESTOCK.DATAENTRADA%TYPE,					   
					   pDATASORTIDA VEHICLESTOCK.DATASORTIDA%TYPE,					   
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pBASTIDOR||';'||pIDCONCESSIONARI||';'||pIDTIPUSVEHICLE||';'||pMATRICULA||';'||pANYMODEL||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||
			pTOTALKM||';'||pDATAMATRICULACIO||';'||pPREU||';'||pDATAENTRADA||';'||pDATASORTIDA;

	INSERT INTO VEHICLESTOCK (BASTIDOR,IDCONCESSIONARI,IDTIPUSVEHICLE,MATRICULA,ANYMODEL,IDCOLOR,IDCONFIGURACIO,TOTALKM,DATAMATRICULACIO,PREU,DATAENTRADA,DATASORTIDA) 
	VALUES (pBASTIDOR,pIDCONCESSIONARI,pIDTIPUSVEHICLE,pMATRICULA,pANYMODEL,pIDCOLOR,pIDCONFIGURACIO,pTOTALKM,pDATAMATRICULACIO,pPREU,SYSDATE,pDATASORTIDA);

	INSERIRLOG ('AFEGIRVEHICLESTOCK',PARAMS,RESPOSTA);
        
    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'El vehicle ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIRVEHICLESTOCK',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error al afegir vehicle stock: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIRVEHICLESTOCK',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARVEHICLESTOCK
--
-- Descripci:          Modifica les dades d'un vehicle en stock 
--
-- Parmetres entrada:
						pIDVEHICLESTOCK VEHICLESTOCK.IDVEHICLESTOCK%TYPE,
						pBASTIDOR VEHICLESTOCK.BASTIDOR%TYPE,
						pIDCONCESSIONARI VEHICLESTOCK.IDCONCESSIONARI%TYPE,
						pIDTIPUSVEHICLE VEHICLESTOCK.IDTIPUSVEHICLE%TYPE,
						pMATRICULA VEHICLESTOCK.MATRICULA%TYPE,
						pANYMODEL VEHICLESTOCK.ANYMODEL%TYPE,
						pIDCOLOR VEHICLESTOCK.IDCOLOR%TYPE,
						pIDCONFIGURACIO VEHICLESTOCK.IDCONFIGURACIO%TYPE,
						pTOTALKM VEHICLESTOCK.TOTALKM%TYPE,
						pDATAMATRICULACIO VEHICLESTOCK.DATAMATRICULACIO%TYPE,
						pPREU VEHICLESTOCK.PREU%TYPE,
						pDATAENTRADA VEHICLESTOCK.DATAENTRADA%TYPE,
						pDATASORTIDA VEHICLESTOCK.DATASORTIDA%TYPE
--
-- Error controlats:   - Vehicle no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARVEHICLESTOCK" ( 
					   pIDVEHICLESTOCK VEHICLESTOCK.IDVEHICLESTOCK%TYPE,
					   pBASTIDOR VEHICLESTOCK.BASTIDOR%TYPE,
					   pIDCONCESSIONARI VEHICLESTOCK.IDCONCESSIONARI%TYPE,
					   pIDTIPUSVEHICLE VEHICLESTOCK.IDTIPUSVEHICLE%TYPE,
					   pMATRICULA VEHICLESTOCK.MATRICULA%TYPE,
					   pANYMODEL VEHICLESTOCK.ANYMODEL%TYPE,
					   pIDCOLOR VEHICLESTOCK.IDCOLOR%TYPE,
					   pIDCONFIGURACIO VEHICLESTOCK.IDCONFIGURACIO%TYPE,
					   pTOTALKM VEHICLESTOCK.TOTALKM%TYPE,
					   pDATAMATRICULACIO VEHICLESTOCK.DATAMATRICULACIO%TYPE,
					   pPREU VEHICLESTOCK.PREU%TYPE,
					   pDATAENTRADA VEHICLESTOCK.DATAENTRADA%TYPE,					   
					   pDATASORTIDA VEHICLESTOCK.DATASORTIDA%TYPE,					   
					   RESPOSTA OUT NOCOPY VARCHAR2)  					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDVEHICLESTOCK||';'||pBASTIDOR||';'||pIDCONCESSIONARI||';'||pIDTIPUSVEHICLE||';'||pMATRICULA||';'||pANYMODEL||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||
			pTOTALKM||';'||pDATAMATRICULACIO||';'||pPREU||';'||pDATAENTRADA||';'||pDATASORTIDA;

	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLESTOCK WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
    UPDATE VEHICLESTOCK
    SET BASTIDOR = pBASTIDOR,
	    IDCONCESSIONARI = pIDCONCESSIONARI, 
		IDTIPUSVEHICLE = pIDTIPUSVEHICLE, 
		MATRICULA = pMATRICULA, 
		ANYMODEL = pANYMODEL, 
		IDCOLOR = pIDCOLOR, 
		IDCONFIGURACIO = pIDCONFIGURACIO, 
		TOTALKM = pTOTALKM, 
		DATAMATRICULACIO = pDATAMATRICULACIO, 
		PREU = pPREU,
		DATAENTRADA = pDATAENTRADA,
		DATASORTIDA = pDATASORTIDA
	WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;		

	INSERIRLOG ('MODIFICARVEHICLESTOCK',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap vehicle amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARVEHICLESTOCK',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del vehicle stock : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARVEHICLESTOCK',PARAMS,RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 ESBORRARVEHICLESTOCK
--
-- Descripci:          Esborra un vehicle en stock
--
-- Parmetres entrada:
						pIDVEHICLESTOCK VEHICLESTOCK.IDVEHICLESTOCK%TYPE
--
-- Error controlats:   - Vehicle no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARVEHICLESTOCK" ( 
					   pIDVEHICLESTOCK VEHICLESTOCK.IDVEHICLESTOCK%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDVEHICLESTOCK;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLESTOCK WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM VEHICLESTOCK WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;

	INSERIRLOG ('ESBORRARVEHICLESTOCK',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap vehicle amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARVEHICLESTOCK',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant vehicle stock : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);   
		  INSERIRLOG ('ESBORRARVEHICLESTOCK',PARAMS,RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 AFEGIRMODELCATALEG
--
-- Descripci:          Afegeix un model al catleg
--
-- Parmetres entrada:
						pANYCATALEG CATALEGVEHICLE.ANYCATALEG%TYPE,
						pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
						pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
						pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
						pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
						pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE,
						pPREU CATALEGVEHICLE.PREU%TYPE,					
						pDATABAIXA CATALEGVEHICLE.DATABAIXA%TYPE
--
-- Error controlats:   - Model duplicat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "AFEGIRMODELCATALEG" ( 
					   pANYCATALEG CATALEGVEHICLE.ANYCATALEG%TYPE,
					   pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
					   pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
					   pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
					   pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
					   pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE,
					   pPREU CATALEGVEHICLE.PREU%TYPE,					
					   pDATABAIXA CATALEGVEHICLE.DATABAIXA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pANYCATALEG||';'||pIDMARCA||';'||pIDMODEL||';'||pIDCOMBUSTIBLE||';'||pIDMOTOR||';'||pIDACABAT||';'||
			pPREU||';'||pDATABAIXA;

	INSERT INTO CATALEGVEHICLE (ANYCATALEG,IDMARCA,IDMODEL,IDCOMBUSTIBLE,IDMOTOR,IDACABAT,PREU,DATAREGISTRE,DATABAIXA) 
	VALUES (pANYCATALEG,pIDMARCA,pIDMODEL,pIDCOMBUSTIBLE,pIDMOTOR,pIDACABAT,pPREU,SYSDATE,pDATABAIXA);
	
	INSERIRLOG ('AFEGIRMODELCATALEG',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'El model ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIRMODELCATALEG',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint el model: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIRMODELCATALEG',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARMODELCATALEG
--
-- Descripci:          Modifica un model del catleg
--
-- Parmetres entrada:
						pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE,
						pANYCATALEG CATALEGVEHICLE.ANYCATALEG%TYPE,
						pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
						pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
						pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
						pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
						pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE,
						pPREU CATALEGVEHICLE.PREU%TYPE,
						pDATAREGISTRE CATALEGVEHICLE.DATAREGISTRE%TYPE,
						pDATABAIXA CATALEGVEHICLE.DATABAIXA%TYPE
--
-- Error controlats:   - Model no trobat al catleg
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARMODELCATALEG" ( 
					   pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE,
					   pANYCATALEG CATALEGVEHICLE.ANYCATALEG%TYPE,
					   pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
					   pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
					   pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
					   pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
					   pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE,
					   pPREU CATALEGVEHICLE.PREU%TYPE,
					   pDATAREGISTRE CATALEGVEHICLE.DATAREGISTRE%TYPE,
					   pDATABAIXA CATALEGVEHICLE.DATABAIXA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 							   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCONFIGURACIO||';'||pANYCATALEG||';'||pIDMARCA||';'||pIDMODEL||';'||pIDCOMBUSTIBLE||';'||pIDMOTOR||';'||
			pIDACABAT||';'||pPREU||';'||pDATAREGISTRE||';'||pDATABAIXA;

	SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
    UPDATE CATALEGVEHICLE
    SET ANYCATALEG = pANYCATALEG, 
		IDMARCA = pIDMARCA, 
		IDMODEL = pIDMODEL, 
		IDCOMBUSTIBLE = pIDCOMBUSTIBLE, 
		IDMOTOR = pIDMOTOR, 
		IDACABAT = pIDACABAT, 
		PREU = pPREU, 
		DATAREGISTRE = pDATAREGISTRE,
		DATABAIXA = pDATABAIXA
	WHERE IDCONFIGURACIO = pIDCONFIGURACIO;

	INSERIRLOG ('MODIFICARMODELCATALEG',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap model amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARMODELCATALEG',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del model : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARMODELCATALEG',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARMODELCATALEG
--
-- Descripci:          Esborra un model del catleg
--
-- Parmetres entrada:
						pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE
--
-- Error controlats:   - Model no trobat al catleg
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARMODELCATALEG" ( 
					   pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCONFIGURACIO;

	SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;

	INSERIRLOG ('ESBORRARMODELCATALEG',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap model amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARMODELCATALEG',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant model : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);   
		  INSERIRLOG ('ESBORRARMODELCATALEG',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 REGISTRARCITATALLER
--
-- Descripci:          Registrar una cita a l'agenda
--
-- Parmetres entrada:
						pIDTALLER AGENDA.IDTALLER%TYPE,
						pDIA AGENDA.DIA%TYPE,
						pHORA AGENDA.HORA%TYPE,
						pIDTIPUSCITA AGENDA.IDTIPUSCITA%TYPE,			
						pIDCLIENT AGENDA.IDCLIENT%TYPE,
						pIDVEHICLECLIENT AGENDA.IDVEHICLECLIENT%TYPE,
						pOBSERVACIONS AGENDA.OBSERVACIONS%TYPE
--
-- Error controlats:    - Cita duplicada
						- Client/vehicle no existeix
						- Taller sense disponibilitat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "REGISTRARCITATALLER" ( 
					   pIDTALLER AGENDA.IDTALLER%TYPE,
					   pDIA AGENDA.DIA%TYPE,
					   pHORA AGENDA.HORA%TYPE,
					   pIDTIPUSCITA AGENDA.IDTIPUSCITA%TYPE,			
					   pIDCLIENT AGENDA.IDCLIENT%TYPE,
					   pIDVEHICLECLIENT AGENDA.IDVEHICLECLIENT%TYPE,
					   pOBSERVACIONS AGENDA.OBSERVACIONS%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  NUM_CITES_TALLER INTEGER;
  NUM_MAXIM_CITES INTEGER;
  SUPERAT_MAXIM_CITES EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDTALLER||';'||pDIA||';'||pHORA||';'||pIDTIPUSCITA||';'||pIDCLIENT||';'||pIDVEHICLECLIENT||';'||
			pOBSERVACIONS;
	
	-- Validem que existeix la relacio propietari (client-vehicle)
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE IDCLIENT = pIDCLIENT AND IDVEHICLECLIENT = pIDVEHICLECLIENT AND DATABAIXA IS NULL;
	IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
	
	-- Validar numero cites maxim del taller
	SELECT COUNT(*) INTO NUM_CITES_TALLER FROM AGENDA WHERE IDTALLER = pIDTALLER AND DIA = pDIA AND HORA = pHORA;
	SELECT NUMCITES INTO NUM_MAXIM_CITES FROM TALLER WHERE IDTALLER = pIDTALLER;
	IF (NUM_CITES_TALLER + 1) > NUM_MAXIM_CITES THEN RAISE SUPERAT_MAXIM_CITES; END IF;
	
	INSERT INTO AGENDA (IDTALLER,DIA,HORA,IDTIPUSCITA,IDCLIENT,IDVEHICLECLIENT,OBSERVACIONS)
	VALUES (pIDTALLER,pDIA,pHORA,pIDTIPUSCITA,pIDCLIENT,pIDVEHICLECLIENT,pOBSERVACIONS);

	INSERIRLOG ('REGISTRARCITATALLER',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'La cita ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARCITATALLER',PARAMS,RESPOSTA);
		  
        WHEN NO_DATA_FOUND THEN
          RESPOSTA := 'No existeix cap client amb aquest vehicle';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARCITATALLER',PARAMS,RESPOSTA);
			  
		WHEN SUPERAT_MAXIM_CITES THEN
          RESPOSTA := 'El taller seleccionat no te disponibilitat per aquest dia i hora';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARCITATALLER',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint la cita: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARCITATALLER',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARCITATALLER
--
-- Descripci:          Modificar una cita de l'agenda
--
-- Parmetres entrada:
						pIDCITA AGENDA.IDCITA%TYPE,
						pIDTALLER AGENDA.IDTALLER%TYPE,
						pDIA AGENDA.DIA%TYPE,
						pHORA AGENDA.HORA%TYPE,
						pIDTIPUSCITA AGENDA.IDTIPUSCITA%TYPE,
						pIDCLIENT AGENDA.IDCLIENT%TYPE,
						pIDVEHICLECLIENT AGENDA.IDVEHICLECLIENT%TYPE,
						pOBSERVACIONS AGENDA.OBSERVACIONS%TYPE						
--
-- Error controlats:    - Cita no trobada
						- Client/vehicle no existeix
						- Taller sense disponibilitat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARCITATALLER" ( 
					   pIDCITA AGENDA.IDCITA%TYPE,
					   pIDTALLER AGENDA.IDTALLER%TYPE,
					   pDIA AGENDA.DIA%TYPE,
					   pHORA AGENDA.HORA%TYPE,
					   pIDTIPUSCITA AGENDA.IDTIPUSCITA%TYPE,
					   pIDCLIENT AGENDA.IDCLIENT%TYPE,
					   pIDVEHICLECLIENT AGENDA.IDVEHICLECLIENT%TYPE,
					   pOBSERVACIONS AGENDA.OBSERVACIONS%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2)  							   
  AS
  COUNT_TMP INTEGER;
  NUM_CITES_TALLER INTEGER;
  NUM_MAXIM_CITES INTEGER;
  PROPIETARI_NOTROBAT EXCEPTION;
  SUPERAT_MAXIM_CITES EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCITA||';'||pIDTALLER||';'||pDIA||';'||pHORA||';'||pIDTIPUSCITA||';'||pIDCLIENT||';'||pIDVEHICLECLIENT||';'||
			pOBSERVACIONS;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM AGENDA WHERE IDCITA = pIDCITA;
	IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
    
	-- Validem que existeix la relacio propietari (client-vehicle)
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE IDCLIENT = pIDCLIENT AND IDVEHICLECLIENT = pIDVEHICLECLIENT AND DATABAIXA IS NULL;
	IF (COUNT_TMP = 0) THEN RAISE PROPIETARI_NOTROBAT; END IF;
	
	-- Validar numero cites maxim del taller
	SELECT COUNT(*) INTO NUM_CITES_TALLER FROM AGENDA WHERE IDTALLER = pIDTALLER AND DIA = pDIA AND HORA = pHORA AND IDCITA <> pIDCITA;
	SELECT NUMCITES INTO NUM_MAXIM_CITES FROM TALLER WHERE IDTALLER = pIDTALLER;
	IF (NUM_CITES_TALLER + 1) > NUM_MAXIM_CITES THEN RAISE SUPERAT_MAXIM_CITES; END IF;
    
    UPDATE AGENDA
    SET IDTALLER = pIDTALLER, 
		DIA = pDIA, 
		HORA = pHORA, 
		IDTIPUSCITA = pIDTIPUSCITA, 		
		IDCLIENT = pIDCLIENT, 
		IDVEHICLECLIENT = pIDVEHICLECLIENT, 
		OBSERVACIONS = pOBSERVACIONS
	WHERE IDCITA = pIDCITA;

	INSERIRLOG ('MODIFICARCITATALLER',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap cita amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARCITATALLER',PARAMS,RESPOSTA);
		   
        WHEN PROPIETARI_NOTROBAT THEN
          RESPOSTA := 'No existeix cap client amb aquest vehicle';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARCITATALLER',PARAMS,RESPOSTA);
		  
		WHEN SUPERAT_MAXIM_CITES THEN
          RESPOSTA := 'El taller seleccionat no te disponibilitat per aquest dia i hora';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARCITATALLER',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci de la cita : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARCITATALLER',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARCITATALLER
--
-- Descripci:          Esborra una cita de l'agenda
--
-- Parmetres entrada:
						pIDCITA AGENDA.IDCITA%TYPE				
--
-- Error controlats:    - Cita no trobada
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARCITATALLER" ( 
					   pIDCITA AGENDA.IDCITA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCITA;

	SELECT COUNT(*) INTO COUNT_TMP FROM AGENDA WHERE IDCITA = pIDCITA;
    IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
    
	DELETE FROM AGENDA WHERE IDCITA = pIDCITA;

	INSERIRLOG ('ESBORRARCITATALLER',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap cita amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARCITATALLER',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant cita : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ESBORRARCITATALLER',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 REGISTRARREPARACIO
--
-- Descripci:          Registra la reparaci d'un vehicle
--
-- Parmetres entrada:
						pBASTIDOR REPARACIO.BASTIDOR%TYPE,
						pKM REPARACIO.KM%TYPE,
						pDATAENTRADA REPARACIO.DATAENTRADA%TYPE,
						pDATALLIURAMENT REPARACIO.DATALLIURAMENT%TYPE,
						pTEMPSRESOLUCIO REPARACIO.TEMPSRESOLUCIO%TYPE,
						pIMPORTMAOBRA REPARACIO.IMPORTMAOBRA%TYPE,
						pIMPORTMATERIAL REPARACIO.IMPORTMATERIAL%TYPE,
						pOBSERVACIONS REPARACIO.OBSERVACIONS%TYPE,
						pIDMECANIC REPARACIO.IDMECANIC%TYPE,
						pIDTALLER REPARACIO.IDTALLER%TYPE,
						pIDCITA REPARACIO.IDCITA%TYPE				
--
-- Error controlats:    - Reparaci duplicada
						- Client/vehicle no existeix
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "REGISTRARREPARACIO" ( 
					   pBASTIDOR REPARACIO.BASTIDOR%TYPE,
					   pKM REPARACIO.KM%TYPE,
					   pDATAENTRADA REPARACIO.DATAENTRADA%TYPE,
					   pDATALLIURAMENT REPARACIO.DATALLIURAMENT%TYPE,
					   pTEMPSRESOLUCIO REPARACIO.TEMPSRESOLUCIO%TYPE,
					   pIMPORTMAOBRA REPARACIO.IMPORTMAOBRA%TYPE,
					   pIMPORTMATERIAL REPARACIO.IMPORTMATERIAL%TYPE,
					   pOBSERVACIONS REPARACIO.OBSERVACIONS%TYPE,
					   pIDMECANIC REPARACIO.IDMECANIC%TYPE,
					   pIDTALLER REPARACIO.IDTALLER%TYPE,
					   pIDCITA REPARACIO.IDCITA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  VEHICLE_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pBASTIDOR||';'||pKM||';'||pDATAENTRADA||';'||pDATALLIURAMENT||';'||pTEMPSRESOLUCIO||';'||pIMPORTMAOBRA||';'||pIMPORTMATERIAL||';'||
			pOBSERVACIONS||';'||pIDMECANIC||';'||pIDTALLER||';'||pIDCITA;
	
	-- Validem que existeix el vehicle
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE BASTIDOR = pBASTIDOR;
	IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
	
	INSERT INTO REPARACIO (BASTIDOR,KM,DATAENTRADA,DATALLIURAMENT,TEMPSRESOLUCIO,IMPORTMAOBRA,IMPORTMATERIAL,OBSERVACIONS,IDMECANIC,IDTALLER,IDCITA)
	VALUES (pBASTIDOR,pKM,pDATAENTRADA,pDATALLIURAMENT,pTEMPSRESOLUCIO,pIMPORTMAOBRA,pIMPORTMATERIAL,pOBSERVACIONS,pIDMECANIC,pIDTALLER,pIDCITA);

	INSERIRLOG ('REGISTRARREPARACIO',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'La reparacio ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARREPARACIO',PARAMS,RESPOSTA);
		  
        WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle amb el bastidor proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARREPARACIO',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint la reparacio: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('REGISTRARREPARACIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARREPARACIO
--
-- Descripci:          Modifica les dades de la reparaci d'un vehicle
--
-- Parmetres entrada:
						pIDREPARACIO REPARACIO.IDREPARACIO%TYPE,
						pBASTIDOR REPARACIO.BASTIDOR%TYPE,
						pKM REPARACIO.KM%TYPE,
						pDATAENTRADA REPARACIO.DATAENTRADA%TYPE,
						pDATALLIURAMENT REPARACIO.DATALLIURAMENT%TYPE,
						pTEMPSRESOLUCIO REPARACIO.TEMPSRESOLUCIO%TYPE,
						pIMPORTMAOBRA REPARACIO.IMPORTMAOBRA%TYPE,
						pIMPORTMATERIAL REPARACIO.IMPORTMATERIAL%TYPE,
						pOBSERVACIONS REPARACIO.OBSERVACIONS%TYPE,
						pIDMECANIC REPARACIO.IDMECANIC%TYPE,
						pIDTALLER REPARACIO.IDTALLER%TYPE,
						pIDCITA REPARACIO.IDCITA%TYPE
--
-- Error controlats:    - Reparaci no trobada
						- Client/vehicle no existeix
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARREPARACIO" ( 
					   pIDREPARACIO REPARACIO.IDREPARACIO%TYPE,
					   pBASTIDOR REPARACIO.BASTIDOR%TYPE,
					   pKM REPARACIO.KM%TYPE,
					   pDATAENTRADA REPARACIO.DATAENTRADA%TYPE,
					   pDATALLIURAMENT REPARACIO.DATALLIURAMENT%TYPE,
					   pTEMPSRESOLUCIO REPARACIO.TEMPSRESOLUCIO%TYPE,
					   pIMPORTMAOBRA REPARACIO.IMPORTMAOBRA%TYPE,
					   pIMPORTMATERIAL REPARACIO.IMPORTMATERIAL%TYPE,					   
					   pOBSERVACIONS REPARACIO.OBSERVACIONS%TYPE,
					   pIDMECANIC REPARACIO.IDMECANIC%TYPE,
					   pIDTALLER REPARACIO.IDTALLER%TYPE,
					   pIDCITA REPARACIO.IDCITA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  VEHICLE_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDREPARACIO||';'||pBASTIDOR||';'||pKM||';'||pDATAENTRADA||';'||pDATALLIURAMENT||';'||pTEMPSRESOLUCIO||';'||pIMPORTMAOBRA||';'||pIMPORTMATERIAL||';'||
			pOBSERVACIONS||';'||pIDMECANIC||';'||pIDTALLER||';'||pIDCITA;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM REPARACIO WHERE IDREPARACIO = pIDREPARACIO;
    IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
	
	-- Validem que existeix el vehicle
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE BASTIDOR = pBASTIDOR;
	IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
    
    UPDATE REPARACIO
    SET BASTIDOR = pBASTIDOR, 
		KM = pKM, 
		DATAENTRADA = pDATAENTRADA, 
		DATALLIURAMENT = pDATALLIURAMENT, 
		TEMPSRESOLUCIO = pTEMPSRESOLUCIO,
		IMPORTMAOBRA = pIMPORTMAOBRA,
		IMPORTMATERIAL = pIMPORTMATERIAL, 
		OBSERVACIONS = pOBSERVACIONS, 
		IDMECANIC = pIDMECANIC, 
		IDTALLER = pIDTALLER, 
		IDCITA = pIDCITA
	WHERE IDREPARACIO = pIDREPARACIO;

	INSERIRLOG ('MODIFICARREPARACIO',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap reparacio amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARREPARACIO',PARAMS,RESPOSTA);
        
		WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle amb el bastidor proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARREPARACIO',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci de la reparacio : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('MODIFICARREPARACIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARREPARACIO
--
-- Descripci:          Esborra la reparaci d'un vehicle
--
-- Parmetres entrada:
						pIDREPARACIO REPARACIO.IDREPARACIO%TYPE
--
-- Error controlats:    - Reparaci no trobada
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARREPARACIO" ( 
					   pIDREPARACIO REPARACIO.IDREPARACIO%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDREPARACIO;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM REPARACIO WHERE IDREPARACIO = pIDREPARACIO;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM REPARACIO WHERE IDREPARACIO = pIDREPARACIO;

	INSERIRLOG ('ESBORRARREPARACIO',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap reparacio amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARREPARACIO',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant reparacio : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ESBORRARREPARACIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 REGISTRARREVISIO
--
-- Descripci:          Registra la revisi d'un vehicle
--
-- Parmetres entrada:
						pBASTIDOR REVISIO.BASTIDOR%TYPE,
						pKM REVISIO.KM%TYPE,
						pDATAENTRADA REVISIO.DATAENTRADA%TYPE,
						pDATALLIURAMENT REVISIO.DATALLIURAMENT%TYPE,
						pHABITACLE REVISIO.HABITACLE%TYPE,
						pEXTERIOR REVISIO.EXTERIOR%TYPE,
						pMOTOR REVISIO.MOTOR%TYPE,
						pBAIXOS REVISIO.BAIXOS%TYPE,
						pIMPORTMAOBRA REVISIO.IMPORTMAOBRA%TYPE,
						pIMPORTMATERIAL REVISIO.IMPORTMATERIAL%TYPE,	
						pOBSERVACIONS REVISIO.OBSERVACIONS%TYPE,
						pIDMECANIC REVISIO.IDMECANIC%TYPE,
						pIDTALLER REVISIO.IDTALLER%TYPE,
						pIDCITA REVISIO.IDCITA%TYPE						
--
-- Error controlats:    - Revisi duplicada
						- Client/vehicle no existeix
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "REGISTRARREVISIO" ( 
					   pBASTIDOR REVISIO.BASTIDOR%TYPE,
					   pKM REVISIO.KM%TYPE,
					   pDATAENTRADA REVISIO.DATAENTRADA%TYPE,
					   pDATALLIURAMENT REVISIO.DATALLIURAMENT%TYPE,
					   pHABITACLE REVISIO.HABITACLE%TYPE,
					   pEXTERIOR REVISIO.EXTERIOR%TYPE,
					   pMOTOR REVISIO.MOTOR%TYPE,
					   pBAIXOS REVISIO.BAIXOS%TYPE,
					   pIMPORTMAOBRA REVISIO.IMPORTMAOBRA%TYPE,
					   pIMPORTMATERIAL REVISIO.IMPORTMATERIAL%TYPE,	
					   pOBSERVACIONS REVISIO.OBSERVACIONS%TYPE,
					   pIDMECANIC REVISIO.IDMECANIC%TYPE,
					   pIDTALLER REVISIO.IDTALLER%TYPE,
					   pIDCITA REVISIO.IDCITA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  VEHICLE_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pBASTIDOR||';'||pKM||';'||pDATAENTRADA||';'||pDATALLIURAMENT||';'||pHABITACLE||';'||pEXTERIOR||';'||pMOTOR||';'||
			pBAIXOS||';'||pIMPORTMAOBRA||';'||pIMPORTMATERIAL||';'||pOBSERVACIONS||';'||pIDMECANIC||';'||pIDTALLER||';'||pIDCITA;
			
	-- Validem que existeix el vehicle
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE BASTIDOR = pBASTIDOR;
	IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
	
	INSERT INTO REVISIO (BASTIDOR,KM,DATAENTRADA,DATALLIURAMENT,HABITACLE,EXTERIOR,MOTOR,BAIXOS,IMPORTMAOBRA,IMPORTMATERIAL,OBSERVACIONS,IDMECANIC,IDTALLER,IDCITA)
	VALUES (pBASTIDOR,pKM,pDATAENTRADA,pDATALLIURAMENT,pHABITACLE,pEXTERIOR,pMOTOR,pBAIXOS,pIMPORTMAOBRA,pIMPORTMATERIAL,pOBSERVACIONS,pIDMECANIC,pIDTALLER,pIDCITA);

	INSERIRLOG ('REGISTRARREVISIO',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'La revisio ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARREVISIO',PARAMS,RESPOSTA);
		  
        WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle amb el bastidor proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REGISTRARREVISIO',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint la revisio: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('REGISTRARREVISIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARREVISIO
--
-- Descripci:          Modifica les dades de la revisi d'un vehicle
--
-- Parmetres entrada:
						pIDREVISIO REVISIO.IDREVISIO%TYPE,
						pBASTIDOR REVISIO.BASTIDOR%TYPE,
						pKM REVISIO.KM%TYPE,
						pDATAENTRADA REVISIO.DATAENTRADA%TYPE,
						pDATALLIURAMENT REVISIO.DATALLIURAMENT%TYPE,
						pHABITACLE REVISIO.HABITACLE%TYPE,
						pEXTERIOR REVISIO.EXTERIOR%TYPE,
						pMOTOR REVISIO.MOTOR%TYPE,
						pBAIXOS REVISIO.BAIXOS%TYPE,
						pIMPORTMAOBRA REVISIO.IMPORTMAOBRA%TYPE,
						pIMPORTMATERIAL REVISIO.IMPORTMATERIAL%TYPE,	
						pOBSERVACIONS REVISIO.OBSERVACIONS%TYPE,
						pIDMECANIC REVISIO.IDMECANIC%TYPE,
						pIDTALLER REVISIO.IDTALLER%TYPE,
						pIDCITA REVISIO.IDCITA%TYPE
--
-- Error controlats:    - Revisi no trobada
						- Client/vehicle no existeix
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARREVISIO" ( 
					   pIDREVISIO REVISIO.IDREVISIO%TYPE,
					   pBASTIDOR REVISIO.BASTIDOR%TYPE,
					   pKM REVISIO.KM%TYPE,
					   pDATAENTRADA REVISIO.DATAENTRADA%TYPE,
					   pDATALLIURAMENT REVISIO.DATALLIURAMENT%TYPE,
					   pHABITACLE REVISIO.HABITACLE%TYPE,
					   pEXTERIOR REVISIO.EXTERIOR%TYPE,
					   pMOTOR REVISIO.MOTOR%TYPE,
					   pBAIXOS REVISIO.BAIXOS%TYPE,
					   pIMPORTMAOBRA REVISIO.IMPORTMAOBRA%TYPE,
					   pIMPORTMATERIAL REVISIO.IMPORTMATERIAL%TYPE,	
					   pOBSERVACIONS REVISIO.OBSERVACIONS%TYPE,
					   pIDMECANIC REVISIO.IDMECANIC%TYPE,
					   pIDTALLER REVISIO.IDTALLER%TYPE,
					   pIDCITA REVISIO.IDCITA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  VEHICLE_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDREVISIO||';'||pBASTIDOR||';'||pKM||';'||pDATAENTRADA||';'||pDATALLIURAMENT||';'||pHABITACLE||';'||pEXTERIOR||';'||pMOTOR||';'||
			pBAIXOS||';'||pIMPORTMAOBRA||';'||pIMPORTMATERIAL||';'||pOBSERVACIONS||';'||pIDMECANIC||';'||pIDTALLER||';'||pIDCITA;

	SELECT COUNT(*) INTO COUNT_TMP FROM REVISIO WHERE IDREVISIO = pIDREVISIO;
    IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
	
	-- Validem que existeix el vehicle
	SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLECLIENT WHERE BASTIDOR = pBASTIDOR;
	IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
    
    UPDATE REVISIO
    SET BASTIDOR = pBASTIDOR, 
		KM = pKM, 
		DATAENTRADA = pDATAENTRADA, 
		DATALLIURAMENT = pDATALLIURAMENT, 
		HABITACLE = pHABITACLE,
		EXTERIOR = pEXTERIOR,
		MOTOR = pMOTOR,
		BAIXOS = pBAIXOS,
		IMPORTMAOBRA = pIMPORTMAOBRA,
		IMPORTMATERIAL = pIMPORTMATERIAL, 
		OBSERVACIONS = pOBSERVACIONS, 
		IDMECANIC = pIDMECANIC, 
		IDTALLER = pIDTALLER, 
		IDCITA = pIDCITA
	WHERE IDREVISIO = pIDREVISIO;

	INSERIRLOG ('MODIFICARREVISIO',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap revisio amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARREVISIO',PARAMS,RESPOSTA);
        
		WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle amb el bastidor proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARREVISIO',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci de la revisio : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
		  INSERIRLOG ('MODIFICARREVISIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARREVISIO
--
-- Descripci:          Esborrar la revisi d'un vehicle
--
-- Parmetres entrada:
						pIDREVISIO REVISIO.IDREVISIO%TYPE
--
-- Error controlats:    - Revisi no trobada
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARREVISIO" ( 
					   pIDREVISIO REVISIO.IDREVISIO%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
   
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDREVISIO;

	SELECT COUNT(*) INTO COUNT_TMP FROM REVISIO WHERE IDREVISIO = pIDREVISIO;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM REVISIO WHERE IDREVISIO = pIDREVISIO;

	INSERIRLOG ('ESBORRARREVISIO',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap revisio amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARREVISIO',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant revisio : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('ESBORRARREVISIO',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 CREARPRESSUPOST
--
-- Descripci:          Crear el pressupost d'un vehicle
--
-- Parmetres entrada:
						pIDCOMERCIAL PRESSUPOST.IDCOMERCIAL%TYPE,
						pIDCLIENT PRESSUPOST.IDCLIENT%TYPE,
						pIDTIPUSVEHICLE PRESSUPOST.IDTIPUSVEHICLE%TYPE,
						pIDVEHICLESTOCK PRESSUPOST.IDVEHICLESTOCK%TYPE,
						pIDCOLOR PRESSUPOST.IDCOLOR%TYPE,
						pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
						pPREU PRESSUPOST.PREU%TYPE,
						pDATAPRESSUPOST PRESSUPOST.DATAPRESSUPOST%TYPE
--
-- Error controlats:    - Pressupost duplicat
						- Client/vehicle no existeix
						- Model no catalogat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "CREARPRESSUPOST" ( 
					   pIDCOMERCIAL PRESSUPOST.IDCOMERCIAL%TYPE,
					   pIDCLIENT PRESSUPOST.IDCLIENT%TYPE,
					   pIDTIPUSVEHICLE PRESSUPOST.IDTIPUSVEHICLE%TYPE,
					   pIDVEHICLESTOCK PRESSUPOST.IDVEHICLESTOCK%TYPE,
					   pIDCOLOR PRESSUPOST.IDCOLOR%TYPE,
					   pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
					   pPREU PRESSUPOST.PREU%TYPE,
					   pDATAPRESSUPOST PRESSUPOST.DATAPRESSUPOST%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  CLIENT_NOTROBAT EXCEPTION;
  VEHICLE_NOTROBAT EXCEPTION;
  MODELCATALEG_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCOMERCIAL||';'||pIDCLIENT||';'||pIDTIPUSVEHICLE||';'||pIDVEHICLESTOCK||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||
			pPREU||';'||pDATAPRESSUPOST;

	-- Validem que existeix el client
	SELECT COUNT(*) INTO COUNT_TMP FROM CLIENT WHERE IDCLIENT = pIDCLIENT AND DATABAIXA IS NULL;
	IF (COUNT_TMP = 0) THEN RAISE CLIENT_NOTROBAT; END IF;
	
	-- Validem que existeix el vehicle en stock
	IF (pIDVEHICLESTOCK IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLESTOCK WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;
		IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
	END IF;

	-- Validem que existeix el model configurat al cataleg
	IF (pIDCONFIGURACIO IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
		IF (COUNT_TMP = 0) THEN RAISE MODELCATALEG_NOTROBAT; END IF;
	END IF;

	INSERT INTO PRESSUPOST (IDCOMERCIAL,IDCLIENT,IDTIPUSVEHICLE,IDVEHICLESTOCK,IDCOLOR,IDCONFIGURACIO,PREU,DATAPRESSUPOST) 
	VALUES (pIDCOMERCIAL,pIDCLIENT,pIDTIPUSVEHICLE,pIDVEHICLESTOCK,pIDCOLOR,pIDCONFIGURACIO,pPREU,SYSDATE) ;
        
	INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'El pressupost ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);
        
		WHEN CLIENT_NOTROBAT THEN
          RESPOSTA := 'No existeix cap client amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);
		  
		WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle en stock amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);
		  
		WHEN MODELCATALEG_NOTROBAT THEN
          RESPOSTA := 'No existeix cap model en el cataleg amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error creant pressupost: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);   
		  INSERIRLOG ('CREARPRESSUPOST',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARPRESSUPOST
--
-- Descripci:          Modifica el pressupost d'un vehicle
--
-- Parmetres entrada:
						pIDPRESSUPOST PRESSUPOST.IDPRESSUPOST%TYPE,
						pIDCOMERCIAL PRESSUPOST.IDCOMERCIAL%TYPE,
						pIDCLIENT PRESSUPOST.IDCLIENT%TYPE,
						pIDTIPUSVEHICLE PRESSUPOST.IDTIPUSVEHICLE%TYPE,
						pIDVEHICLESTOCK PRESSUPOST.IDVEHICLESTOCK%TYPE,
						pIDCOLOR PRESSUPOST.IDCOLOR%TYPE,
						pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
						pPREU PRESSUPOST.PREU%TYPE,
						pDATAPRESSUPOST PRESSUPOST.DATAPRESSUPOST%TYPE
--
-- Error controlats:    - Pressupost no trobat
						- Client/vehicle no existeix
						- Model no catalogat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARPRESSUPOST" ( 
					   pIDPRESSUPOST PRESSUPOST.IDPRESSUPOST%TYPE,
					   pIDCOMERCIAL PRESSUPOST.IDCOMERCIAL%TYPE,
					   pIDCLIENT PRESSUPOST.IDCLIENT%TYPE,
					   pIDTIPUSVEHICLE PRESSUPOST.IDTIPUSVEHICLE%TYPE,
					   pIDVEHICLESTOCK PRESSUPOST.IDVEHICLESTOCK%TYPE,
					   pIDCOLOR PRESSUPOST.IDCOLOR%TYPE,
					   pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
					   pPREU PRESSUPOST.PREU%TYPE,
					   pDATAPRESSUPOST PRESSUPOST.DATAPRESSUPOST%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  CLIENT_NOTROBAT EXCEPTION;
  VEHICLE_NOTROBAT EXCEPTION;
  MODELCATALEG_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDPRESSUPOST||';'||pIDCOMERCIAL||';'||pIDCLIENT||';'||pIDTIPUSVEHICLE||';'||pIDVEHICLESTOCK||';'||pIDCOLOR||';'||pIDCONFIGURACIO||';'||
			pPREU||';'||pDATAPRESSUPOST;

	SELECT COUNT(*) INTO COUNT_TMP FROM PRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST;
    IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
	
	-- Validem que existeix el client
	SELECT COUNT(*) INTO COUNT_TMP FROM CLIENT WHERE IDCLIENT = pIDCLIENT AND DATABAIXA IS NULL;
	IF (COUNT_TMP = 0) THEN RAISE CLIENT_NOTROBAT; END IF;
	
	-- Validem que existeix el vehicle en stock
	IF (pIDVEHICLESTOCK IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM VEHICLESTOCK WHERE IDVEHICLESTOCK = pIDVEHICLESTOCK;
		IF (COUNT_TMP = 0) THEN RAISE VEHICLE_NOTROBAT; END IF;
	END IF;

	-- Validem que existeix el model configurat al cataleg
	IF (pIDCONFIGURACIO IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
		IF (COUNT_TMP = 0) THEN RAISE MODELCATALEG_NOTROBAT; END IF;
	END IF;
	
    UPDATE PRESSUPOST
    SET IDCOMERCIAL = pIDCOMERCIAL, 
		IDCLIENT = pIDCLIENT, 
		IDTIPUSVEHICLE = pIDTIPUSVEHICLE, 
		IDVEHICLESTOCK = pIDVEHICLESTOCK,
		IDCOLOR = pIDCOLOR,
		IDCONFIGURACIO = pIDCONFIGURACIO,
		PREU = pPREU,
		DATAPRESSUPOST = pDATAPRESSUPOST
	WHERE IDPRESSUPOST = pIDPRESSUPOST;

	INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap pressupost amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);

		WHEN CLIENT_NOTROBAT THEN
          RESPOSTA := 'No existeix cap client amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);
		  
		WHEN VEHICLE_NOTROBAT THEN
          RESPOSTA := 'No existeix cap vehicle en stock amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);
		  
		WHEN MODELCATALEG_NOTROBAT THEN
          RESPOSTA := 'No existeix cap model en el cataleg amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del pressupost : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARPRESSUPOST',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARPRESSUPOST
--
-- Descripci:          Esborra el pressupost d'un vehicle
--
-- Parmetres entrada:
						pIDPRESSUPOST PRESSUPOST.IDPRESSUPOST%TYPE
--
-- Error controlats:    - Pressupost no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARPRESSUPOST" ( 
					   pIDPRESSUPOST PRESSUPOST.IDPRESSUPOST%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDPRESSUPOST;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM PRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM PRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST;

	INSERIRLOG ('ESBORRARPRESSUPOST',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap pressupost amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARPRESSUPOST',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant pressupost : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ESBORRARPRESSUPOST',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 REALITZARCOMANDAFABRICA
--
-- Descripci:          Realitzar la comanda d'un vehicle nou a fbrica
--
-- Parmetres entrada:
						pIDCONCESSIONARI COMANDAFABRICA.IDCONCESSIONARI%TYPE,
						pDATACOMANDA COMANDAFABRICA.DATACOMANDA%TYPE,
						pIDVENDA COMANDAFABRICA.IDVENDA%TYPE,
						pIDCONFIGURACIO COMANDAFABRICA.IDCONFIGURACIO%TYPE,
						pIDCOLOR COMANDAFABRICA.IDCOLOR%TYPE,
						pSETMANAPROD COMANDAFABRICA.SETMANAPROD%TYPE,
						pFIPRODUCCIO COMANDAFABRICA.FIPRODUCCIO%TYPE,
						pBASTIDOR COMANDAFABRICA.BASTIDOR%TYPE
--
-- Error controlats:    - Comanda duplicada
						- Model no catalogat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "REALITZARCOMANDAFABRICA" ( 
					   pIDCONCESSIONARI COMANDAFABRICA.IDCONCESSIONARI%TYPE,
					   pDATACOMANDA COMANDAFABRICA.DATACOMANDA%TYPE,
					   pIDVENDA COMANDAFABRICA.IDVENDA%TYPE,
					   pIDCONFIGURACIO COMANDAFABRICA.IDCONFIGURACIO%TYPE,
					   pIDCOLOR COMANDAFABRICA.IDCOLOR%TYPE,
					   pSETMANAPROD COMANDAFABRICA.SETMANAPROD%TYPE,
					   pFIPRODUCCIO COMANDAFABRICA.FIPRODUCCIO%TYPE,
					   pBASTIDOR COMANDAFABRICA.BASTIDOR%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  MODELCATALEG_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCONCESSIONARI||';'||pDATACOMANDA||';'||pIDVENDA||';'||pIDCONFIGURACIO||';'||pIDCOLOR||';'||pSETMANAPROD||';'||
			pFIPRODUCCIO||';'||pBASTIDOR;
			
	-- Validem que existeix el model configurat al cataleg
	IF (pIDCONFIGURACIO IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
		IF (COUNT_TMP = 0) THEN RAISE MODELCATALEG_NOTROBAT; END IF;
	END IF;

	INSERT INTO COMANDAFABRICA (IDCONCESSIONARI,DATACOMANDA,IDVENDA,IDCONFIGURACIO,IDCOLOR,SETMANAPROD,FIPRODUCCIO,BASTIDOR) 
	VALUES (pIDCONCESSIONARI,pDATACOMANDA,pIDVENDA,pIDCONFIGURACIO,pIDCOLOR,pSETMANAPROD,pFIPRODUCCIO,pBASTIDOR);
        
	INSERIRLOG ('REALITZARCOMANDAFABRICA',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'La comanda ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REALITZARCOMANDAFABRICA',PARAMS,RESPOSTA);
        
		WHEN MODELCATALEG_NOTROBAT THEN
          RESPOSTA := 'No existeix cap model en el cataleg amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REALITZARCOMANDAFABRICA',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error creant comanda: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  INSERIRLOG ('REALITZARCOMANDAFABRICA',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 MODIFICARCOMANDAFABRICA
--
-- Descripci:          Modifica la comanda d'un vehicle nou a fbrica
--
-- Parmetres entrada:
						pIDCOMANDA COMANDAFABRICA.IDCOMANDA%TYPE,
						pIDCONCESSIONARI COMANDAFABRICA.IDCONCESSIONARI%TYPE,
						pDATACOMANDA COMANDAFABRICA.DATACOMANDA%TYPE,
						pIDVENDA COMANDAFABRICA.IDVENDA%TYPE,
						pIDCONFIGURACIO COMANDAFABRICA.IDCONFIGURACIO%TYPE,
						pIDCOLOR COMANDAFABRICA.IDCOLOR%TYPE,
						pSETMANAPROD COMANDAFABRICA.SETMANAPROD%TYPE,
						pFIPRODUCCIO COMANDAFABRICA.FIPRODUCCIO%TYPE,
						pBASTIDOR COMANDAFABRICA.BASTIDOR%TYPE
--
-- Error controlats:    - Comanda no trobada
						- Model no catalogat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARCOMANDAFABRICA" ( 
					   pIDCOMANDA COMANDAFABRICA.IDCOMANDA%TYPE,
					   pIDCONCESSIONARI COMANDAFABRICA.IDCONCESSIONARI%TYPE,
					   pDATACOMANDA COMANDAFABRICA.DATACOMANDA%TYPE,
					   pIDVENDA COMANDAFABRICA.IDVENDA%TYPE,
					   pIDCONFIGURACIO COMANDAFABRICA.IDCONFIGURACIO%TYPE,
					   pIDCOLOR COMANDAFABRICA.IDCOLOR%TYPE,
					   pSETMANAPROD COMANDAFABRICA.SETMANAPROD%TYPE,
					   pFIPRODUCCIO COMANDAFABRICA.FIPRODUCCIO%TYPE,
					   pBASTIDOR COMANDAFABRICA.BASTIDOR%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  MODELCATALEG_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCOMANDA||';'||pIDCONCESSIONARI||';'||pDATACOMANDA||';'||pIDVENDA||';'||pIDCONFIGURACIO||';'||pIDCOLOR||';'||pSETMANAPROD||';'||
			pFIPRODUCCIO||';'||pBASTIDOR;

	SELECT COUNT(*) INTO COUNT_TMP FROM COMANDAFABRICA WHERE IDCOMANDA = pIDCOMANDA;
    IF (COUNT_TMP = 0) THEN RAISE NO_DATA_FOUND; END IF;
	
	-- Validem que existeix el model configurat al cataleg
	IF (pIDCONFIGURACIO IS NOT NULL) THEN
		SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
		IF (COUNT_TMP = 0) THEN RAISE MODELCATALEG_NOTROBAT; END IF;
	END IF;

	UPDATE COMANDAFABRICA
    SET IDCONCESSIONARI = pIDCONCESSIONARI, 
		DATACOMANDA = pDATACOMANDA, 
		IDVENDA = pIDVENDA, 
		IDCONFIGURACIO = pIDCONFIGURACIO,
		IDCOLOR = pIDCOLOR,
		SETMANAPROD = pSETMANAPROD,
		FIPRODUCCIO = pFIPRODUCCIO,
		BASTIDOR = pBASTIDOR
	WHERE IDCOMANDA = pIDCOMANDA;
	
	INSERIRLOG ('MODIFICARCOMANDAFABRICA',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap comanda amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('MODIFICARCOMANDAFABRICA',PARAMS,RESPOSTA);

		WHEN MODELCATALEG_NOTROBAT THEN
          RESPOSTA := 'No existeix cap model en el cataleg amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARCOMANDAFABRICA',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error modificant la comanda : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('MODIFICARCOMANDAFABRICA',PARAMS,RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 REALITZARVENDA
--
-- Descripci:          Realitza la venda d'un vehicle
--
-- Parmetres entrada:
						pIDCOMERCIAL VENDA.IDCOMERCIAL%TYPE,
						pIMPORT VENDA.IMPORT%TYPE,
						pDATAVENDA VENDA.DATAVENDA%TYPE,
						pDATALLIURAMENT VENDA.DATALLIURAMENT%TYPE,
						pIDPRESSUPOST VENDA.IDPRESSUPOST%TYPE
--
-- Error controlats:    - Venda duplicada
						- Pressupost no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "REALITZARVENDA" ( 
					   pIDCOMERCIAL VENDA.IDCOMERCIAL%TYPE,
					   pIMPORT VENDA.IMPORT%TYPE,
					   pDATAVENDA VENDA.DATAVENDA%TYPE,
					   pDATALLIURAMENT VENDA.DATALLIURAMENT%TYPE,
					   pIDPRESSUPOST VENDA.IDPRESSUPOST%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PRESSUPOST_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCOMERCIAL||';'||pIMPORT||';'||';'||pDATAVENDA||';'||pDATALLIURAMENT||';'||pIDPRESSUPOST;
	
	-- Validem que existeix el pressupost corresponent
	SELECT COUNT(*) INTO COUNT_TMP FROM PRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST;
	IF (COUNT_TMP = 0) THEN RAISE PRESSUPOST_NOTROBAT; END IF;

	INSERT INTO VENDA (IDCOMERCIAL,IMPORT,DATAVENDA,DATALLIURAMENT,IDPRESSUPOST)
	VALUES (pIDCOMERCIAL,pIMPORT,pDATAVENDA,pDATALLIURAMENT,pIDPRESSUPOST);
        
	INSERIRLOG ('REALITZARVENDA',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'La venda ja existeix.';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REALITZARVENDA',PARAMS,RESPOSTA);
        
		WHEN PRESSUPOST_NOTROBAT THEN
          RESPOSTA := 'No existeix cap pressupost amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REALITZARVENDA',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error creant venda: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('REALITZARVENDA',PARAMS,RESPOSTA);			  
END;
/

/*****************************************************************
-- Nom:                 ESBORRARVENDA
--
-- Descripci:          Esborra la venda d'un vehicle
--
-- Parmetres entrada:
						pIDVENDA VENDA.IDVENDA%TYPE
--
-- Error controlats:    - Venda no trobada
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARVENDA" ( 
					   pIDVENDA VENDA.IDVENDA%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDVENDA;

	SELECT COUNT(*) INTO COUNT_TMP FROM VENDA WHERE IDVENDA = pIDVENDA;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM VENDA WHERE IDVENDA = pIDVENDA;

	INSERIRLOG ('ESBORRARVENDA',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap venda amb el codi proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('ESBORRARVENDA',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant venda : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('ESBORRARVENDA',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 AFEGIROPCIONALPRESSUPOST
--
-- Descripci:          Afegeix un equipament opcional al pressupost
--
-- Parmetres entrada:
						pIDPRESSUPOST OPCIONALPRESSUPOST.IDPRESSUPOST%TYPE,
					    pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE
--
-- Error controlats:    - Opcional duplicat
						- Pressupost no trobat
						- Opcional no catalogat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "AFEGIROPCIONALPRESSUPOST" ( 
					   pIDPRESSUPOST OPCIONALPRESSUPOST.IDPRESSUPOST%TYPE,
					   pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PRESSUPOST_NOTROBAT EXCEPTION;
  OPCIONAL_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDPRESSUPOST||';'||pIDOPCIONAL;
			
	-- Validem que existeix el pressupost
	SELECT COUNT(*) INTO COUNT_TMP FROM PRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST;
	IF (COUNT_TMP = 0) THEN RAISE PRESSUPOST_NOTROBAT; END IF;

	-- Validem que existeix el opcional
	SELECT COUNT(*) INTO COUNT_TMP FROM OPCIONAL WHERE IDOPCIONAL = pIDOPCIONAL;
	IF (COUNT_TMP = 0) THEN RAISE OPCIONAL_NOTROBAT; END IF;
	
	INSERT INTO OPCIONALPRESSUPOST (IDPRESSUPOST,IDOPCIONAL) 
	VALUES (pIDPRESSUPOST,pIDOPCIONAL);
        
	INSERIRLOG ('AFEGIROPCIONALPRESSUPOST',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'Ja existeix l''opcional per aquest pressupost';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALPRESSUPOST',PARAMS,RESPOSTA);
        
		WHEN PRESSUPOST_NOTROBAT THEN
          RESPOSTA := 'No existeix cap pressupost amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALPRESSUPOST',PARAMS,RESPOSTA);
		  
		WHEN OPCIONAL_NOTROBAT THEN
          RESPOSTA := 'No existeix cap opcional amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALPRESSUPOST',PARAMS,RESPOSTA);		  
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint opcional al pressupost: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALPRESSUPOST',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 TREUREOPCIONALPRESSUPOST
--
-- Descripci:          Treu un equipament opcional al pressupost
--
-- Parmetres entrada:
						pIDPRESSUPOST OPCIONALPRESSUPOST.IDPRESSUPOST%TYPE,
					    pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE
--
-- Error controlats:    - Opcional no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "TREUREOPCIONALPRESSUPOST" ( 
					   pIDPRESSUPOST OPCIONALPRESSUPOST.IDPRESSUPOST%TYPE,
					   pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDPRESSUPOST||';'||pIDOPCIONAL;

	SELECT COUNT(*) INTO COUNT_TMP FROM OPCIONALPRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST AND IDOPCIONAL = pIDOPCIONAL;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM OPCIONALPRESSUPOST WHERE IDPRESSUPOST = pIDPRESSUPOST AND IDOPCIONAL = pIDOPCIONAL;

	INSERIRLOG ('TREUREOPCIONALPRESSUPOST',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix l''opcional per aquest pressupost';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('TREUREOPCIONALPRESSUPOST',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant opcional al pressupost : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  INSERIRLOG ('TREUREOPCIONALPRESSUPOST',PARAMS,RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 AFEGIROPCIONALCOMANDA
--
-- Descripci:          Afegeix un equipament opcional a la comanda
--
-- Parmetres entrada:
						pIDCOMANDA OPCIONALCOMANDA.IDCOMANDA%TYPE,
					    pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE
--
-- Error controlats:    - Opcional duplicat
						- Comanda no trobada
						- Opcional no catalogat
--

-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "AFEGIROPCIONALCOMANDA" ( 
					   pIDCOMANDA OPCIONALCOMANDA.IDCOMANDA%TYPE,
					   pIDOPCIONAL OPCIONALCOMANDA.IDOPCIONAL%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  COMANDA_NOTROBAT EXCEPTION;
  OPCIONAL_NOTROBAT EXCEPTION;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCOMANDA||';'||pIDOPCIONAL;

	-- Validem que existeix la comanda
	SELECT COUNT(*) INTO COUNT_TMP FROM COMANDAFABRICA WHERE IDCOMANDA = pIDCOMANDA;
	IF (COUNT_TMP = 0) THEN RAISE COMANDA_NOTROBAT; END IF;

	-- Validem que existeix el opcional
	SELECT COUNT(*) INTO COUNT_TMP FROM OPCIONAL WHERE IDOPCIONAL = pIDOPCIONAL;
	IF (COUNT_TMP = 0) THEN RAISE OPCIONAL_NOTROBAT; END IF;
	
	INSERT INTO OPCIONALCOMANDA (IDCOMANDA,IDOPCIONAL) 
	VALUES (pIDCOMANDA,pIDOPCIONAL);
        
	INSERIRLOG ('AFEGIROPCIONALCOMANDA',PARAMS,RESPOSTA);

    EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'Ja existeix l''opcional per aquesta comanda';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALCOMANDA',PARAMS,RESPOSTA);
        
		WHEN COMANDA_NOTROBAT THEN
          RESPOSTA := 'No existeix cap comanda amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALCOMANDA',PARAMS,RESPOSTA);
		  
		WHEN OPCIONAL_NOTROBAT THEN
          RESPOSTA := 'No existeix cap opcional amb el codi proporcionat';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALCOMANDA',PARAMS,RESPOSTA);		  
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error afegint opcional a la comanda: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('AFEGIROPCIONALCOMANDA',PARAMS,RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 TREUREOPCIONALCOMANDA
--
-- Descripci:          Treu un equipament opcional a la comanda
--
-- Parmetres entrada:
						pIDCOMANDA OPCIONALCOMANDA.IDCOMANDA%TYPE,
					    pIDOPCIONAL OPCIONALPRESSUPOST.IDOPCIONAL%TYPE
--
-- Error controlats:    - Opcional no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "TREUREOPCIONALCOMANDA" ( 
					   pIDCOMANDA OPCIONALCOMANDA.IDCOMANDA%TYPE,
					   pIDOPCIONAL OPCIONALCOMANDA.IDOPCIONAL%TYPE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pIDCOMANDA||';'||pIDOPCIONAL;

	SELECT COUNT(*) INTO COUNT_TMP FROM OPCIONALCOMANDA WHERE IDCOMANDA = pIDCOMANDA AND IDOPCIONAL = pIDOPCIONAL;

    IF (COUNT_TMP = 0) THEN
        RAISE NO_DATA_FOUND;
    END IF;
    
	DELETE FROM OPCIONALCOMANDA WHERE IDCOMANDA = pIDCOMANDA AND IDOPCIONAL = pIDOPCIONAL;

	INSERIRLOG ('TREUREOPCIONALCOMANDA',PARAMS,RESPOSTA);

	EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix l''opcional per aquesta comanda';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   INSERIRLOG ('TREUREOPCIONALCOMANDA',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant opcional a la comanda : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  INSERIRLOG ('TREUREOPCIONALCOMANDA',PARAMS,RESPOSTA);
END;
/
