
/*==============================================================*/
/* Drop types				                                    */
/*==============================================================*/
DROP TYPE ConfiguracioList;
DROP TYPE configuracio_obj;
DROP TYPE ColorList;
DROP TYPE color_obj;
DROP TYPE OpcionalList;
DROP TYPE opcional_obj;
DROP TYPE VehiclesRevisioList;
DROP TYPE vehicle_revisio_obj;


/*****************************************************************
-- Nom:                 CalcularPreuModelConfigurat
--
-- Descripci:          Calcula el preu del model configurat
--
-- Parmetres entrada:
						pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
					    pIDCOLOR PRESSUPOST.IDCOLOR%TYPE
--
-- Retorna:
						Preu del model configurat
--						
-- Error controlats:    - Model no catalogat
						- Color no trobat
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE FUNCTION CalcularPreuModelConfigurat ( 
					   pIDCONFIGURACIO PRESSUPOST.IDCONFIGURACIO%TYPE,
					   pIDCOLOR PRESSUPOST.IDCOLOR%TYPE)
					   RETURN NUMBER
  IS
  COUNT_TMP INTEGER;
  PREU_CONFIG NUMBER;
  PREU_COLOR NUMBER;
  PREU_TOTAL NUMBER;
  MODELCATALEG_NOTROBAT EXCEPTION;
  COLOR_NOTROBAT EXCEPTION;
  
  BEGIN
    PREU_TOTAL:=NULL;

	-- Validem que existeix el model configurat
	SELECT COUNT(*) INTO COUNT_TMP FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
	IF (COUNT_TMP = 0) THEN RAISE MODELCATALEG_NOTROBAT; END IF;
	
	-- Validem que existeix el color
	SELECT COUNT(*) INTO COUNT_TMP FROM COLOR WHERE IDCOLOR = pIDCOLOR;
	IF (COUNT_TMP = 0) THEN RAISE COLOR_NOTROBAT; END IF;

	-- Obtenim el preu del model configurat
	SELECT PREU INTO PREU_CONFIG FROM CATALEGVEHICLE WHERE IDCONFIGURACIO = pIDCONFIGURACIO;
	
	-- Obtenim el preu del color segons el model seleccionat
	SELECT CM.PREU INTO PREU_COLOR
	FROM COLORMODEL CM, CATALEGVEHICLE CV
	WHERE CV.IDCONFIGURACIO = pIDCONFIGURACIO
	AND CV.IDMODEL = CM.IDMODEL
	AND CM.IDCOLOR = pIDCOLOR;
	
	PREU_TOTAL:= PREU_CONFIG + PREU_COLOR;
	
	RETURN PREU_TOTAL;
	
    EXCEPTION       
		WHEN MODELCATALEG_NOTROBAT THEN
          DBMS_OUTPUT.PUT_LINE('No existeix cap model en el cataleg amb el codi proporcionat');
  
		WHEN COLOR_NOTROBAT THEN
          DBMS_OUTPUT.PUT_LINE('No existeix cap color amb el codi proporcionat');
		  		  
        WHEN OTHERS THEN
          DBMS_OUTPUT.PUT_LINE('Error calculant preu del model configurat: (' || SQLCODE || ') ' || SQLERRM);	  
END;
/

CREATE OR REPLACE TYPE configuracio_obj AS OBJECT (
  IDCONFIGURACIO    NUMBER, 
  IDMARCA    		NUMBER, 
  IDMODEL           NUMBER, 
  IDCOMBUSTIBLE    	VARCHAR2(2),
  IDMOTOR           NUMBER, 
  IDACABAT     		NUMBER
);
/

CREATE OR REPLACE TYPE ConfiguracioList AS TABLE OF configuracio_obj;
/

/*****************************************************************
-- Nom:                 ConsultarConfiguracionsCataleg
--
-- Descripci:          Consulta els models per id, marca, model, tipus de combustible,	motor i acabat
--
-- Parmetres entrada:
						pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE,
						pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
						pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
						pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
						pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
						pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE						
--
-- Retorna:
						Llista de models coincidents
--						
-- Error controlats:    
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE FUNCTION ConsultarConfiguracionsCataleg ( 
					   pIDCONFIGURACIO CATALEGVEHICLE.IDCONFIGURACIO%TYPE,
					   pIDMARCA CATALEGVEHICLE.IDMARCA%TYPE,
					   pIDMODEL CATALEGVEHICLE.IDMODEL%TYPE,
					   pIDCOMBUSTIBLE CATALEGVEHICLE.IDCOMBUSTIBLE%TYPE,
					   pIDMOTOR CATALEGVEHICLE.IDMOTOR%TYPE,
					   pIDACABAT CATALEGVEHICLE.IDACABAT%TYPE) 
					   RETURN ConfiguracioList
  AS
  lista ConfiguracioList;
  query_string VARCHAR2(500);
  cursorConfiguracions sys_refcursor;
  configuracio CATALEGVEHICLE%ROWTYPE;  
  
  BEGIN
    lista:= ConfiguracioList();
	query_string := 'SELECT * FROM CATALEGVEHICLE WHERE DATABAIXA IS NULL ';	
	IF (pIDCONFIGURACIO IS NOT NULL) THEN query_string := query_string || ' AND IDCONFIGURACIO='||pIDCONFIGURACIO; END IF;
	IF (pIDMARCA IS NOT NULL) THEN query_string := query_string || ' AND IDMARCA='||pIDMARCA; END IF;
	IF (pIDMODEL IS NOT NULL) THEN query_string := query_string || ' AND IDMODEL='||pIDMODEL; END IF;
	IF (pIDCOMBUSTIBLE IS NOT NULL) THEN query_string := query_string || ' AND IDCOMBUSTIBLE='''||pIDCOMBUSTIBLE||''''; END IF;
	IF (pIDMOTOR IS NOT NULL) THEN query_string := query_string || ' AND IDMOTOR='||pIDMOTOR; END IF;
	IF (pIDACABAT IS NOT NULL) THEN query_string := query_string || ' AND IDACABAT='||pIDACABAT; END IF;
	--DBMS_OUTPUT.PUT_LINE('query: '||query_string);
	
	OPEN cursorConfiguracions FOR query_string;
    LOOP
        FETCH cursorConfiguracions INTO configuracio;
        EXIT WHEN cursorConfiguracions%NOTFOUND;
		lista.extend;
		lista(lista.count) := configuracio_obj(configuracio.IDCONFIGURACIO,configuracio.IDMARCA,configuracio.IDMODEL,configuracio.IDCOMBUSTIBLE,configuracio.IDMOTOR,configuracio.IDACABAT);
    END LOOP;
    CLOSE cursorConfiguracions;

	RETURN lista;
	
	EXCEPTION  
        WHEN OTHERS THEN
          DBMS_OUTPUT.PUT_LINE('Error consultant configuracions : (' || SQLCODE || ') ' || SQLERRM);     
END;
/


CREATE OR REPLACE TYPE color_obj AS OBJECT (
  IDCOLOR    	VARCHAR2(4), 
  DESCRIPCIO   	VARCHAR2(50),
  PREU        	NUMBER
);
/

CREATE OR REPLACE TYPE ColorList AS TABLE OF color_obj;
/

/*****************************************************************
-- Nom:                 ConsultarColorsModel
--
-- Descripci:          Consulta els colors disponibles per un model
--
-- Parmetres entrada:
						pIDMODEL MODEL.IDMODEL%TYPE,
					    pIDCOLOR COLOR.IDCOLOR%TYPE						
--
-- Retorna:
						Llista de colors disponibles per el model
--						
-- Error controlats:    
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE FUNCTION ConsultarColorsModel ( 
					   pIDMODEL MODEL.IDMODEL%TYPE,
					   pIDCOLOR COLOR.IDCOLOR%TYPE) 
					   RETURN ColorList
  AS
  lista ColorList;
  query_string VARCHAR2(500);
  cursorColors sys_refcursor;
  idcolor VARCHAR2(4);
  descripcio VARCHAR2(100);
  preu NUMBER;
  
  BEGIN
    lista:= ColorList();
	query_string := 'SELECT C.IDCOLOR, C.DESCRIPCIO, CM.PREU FROM COLOR C, COLORMODEL CM
					 WHERE C.IDCOLOR = CM.IDCOLOR AND CM.IDMODEL ='|| pIDMODEL;
	
	IF (pIDCOLOR IS NOT NULL) THEN query_string := query_string || ' AND CM.IDCOLOR = '''||pIDCOLOR||''''; END IF;
	DBMS_OUTPUT.PUT_LINE('query: '||query_string);
	OPEN cursorColors FOR query_string;
    LOOP
        FETCH cursorColors INTO idcolor,descripcio,preu;
        EXIT WHEN cursorColors%NOTFOUND;
		lista.extend;
		lista(lista.count) := color_obj(idcolor,descripcio,preu);
    END LOOP;
    CLOSE cursorColors;

	RETURN lista;
	
	EXCEPTION  
        WHEN OTHERS THEN
          DBMS_OUTPUT.PUT_LINE('Error consultant colors : (' || SQLCODE || ') ' || SQLERRM);     
END;
/


CREATE OR REPLACE TYPE opcional_obj AS OBJECT (
  IDOPCIONAL    VARCHAR2(3), 
  DESCRIPCIO   	VARCHAR2(100),
  PREU        NUMBER
);
/

CREATE OR REPLACE TYPE OpcionalList AS TABLE OF opcional_obj;
/

/*****************************************************************
-- Nom:                 ConsultarOpcionalsModel
--
-- Descripci:          Consulta els opcionals disponibles per un model
--
-- Parmetres entrada:
						pIDMODEL MODEL.IDMODEL%TYPE,
					    pIDOPCIONAL OPCIONAL.IDOPCIONAL%TYPE						
--
-- Retorna:
						Llista d'opcionals disponibles per el model
--						
-- Error controlats:    
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE FUNCTION ConsultarOpcionalsModel ( 
					   pIDMODEL MODEL.IDMODEL%TYPE,
					   pIDOPCIONAL OPCIONAL.IDOPCIONAL%TYPE) 
					   RETURN OpcionalList
  AS
  lista OpcionalList;
  query_string VARCHAR2(500);
  cursorOpcionals sys_refcursor;
  idopcional VARCHAR2(3);
  descripcio VARCHAR2(100);
  preu NUMBER;
  
  BEGIN
    lista:= OpcionalList();
	query_string := 'SELECT O.IDOPCIONAL, O.DESCRIPCIO, OM.PREU FROM OPCIONAL O, OPCIONALMODEL OM 
					 WHERE O.IDOPCIONAL = OM.IDOPCIONAL AND OM.IDMODEL ='|| pIDMODEL;
	
	IF (pIDOPCIONAL IS NOT NULL) THEN query_string := query_string || ' AND OM.IDOPCIONAL = '''||pIDOPCIONAL||''''; END IF;
	
	OPEN cursorOpcionals FOR query_string;
    LOOP
        FETCH cursorOpcionals INTO idopcional,descripcio,preu;
        EXIT WHEN cursorOpcionals%NOTFOUND;
		lista.extend;
		lista(lista.count) := opcional_obj(idopcional,descripcio,preu);
    END LOOP;
    CLOSE cursorOpcionals;

	RETURN lista;
	
	EXCEPTION  
        WHEN OTHERS THEN
          DBMS_OUTPUT.PUT_LINE('Error consultant opcionals : (' || SQLCODE || ') ' || SQLERRM);     
END;
/

CREATE OR REPLACE TYPE vehicle_revisio_obj AS OBJECT (
  IDVEHICLECLIENT    NUMBER,
  MATRICULA			 VARCHAR2(9),
  DATAULTIMAREVISIO  DATE,
  NOM	   			 VARCHAR2(30),
  COGNOM1  			 VARCHAR2(30),
  COGNOM2  			 VARCHAR2(30),
  ADRECA  			 VARCHAR2(100),
  POBLACIO 			 VARCHAR2(50),
  CODIPOSTAL		 VARCHAR2(5),
  TELEFON			 VARCHAR2(10),
  MAIL				 VARCHAR2(50)
);
/

CREATE OR REPLACE TYPE VehiclesRevisioList AS TABLE OF vehicle_revisio_obj;
/

/*****************************************************************
-- Nom:                 VehiclesPendentsRevisio
--
-- Descripci:          Consulta els vehicles que van passar la revisi fa 12 mesos
--
-- Parmetres entrada:					
--
-- Retorna:
						Llista de vehicles pendents de passar la revisi
--						
-- Error controlats:    
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE FUNCTION VehiclesPendentsRevisio 
					   RETURN VehiclesRevisioList
  AS
  lista              VehiclesRevisioList;
  query_string       VARCHAR2(500);
  cursorVehicles     sys_refcursor;
  idvehicleclient    NUMBER;
  matricula			 VARCHAR2(9);
  dataultimarevisio  DATE;
  nom	   			 VARCHAR2(30);
  cognom1  			 VARCHAR2(30);
  cognom2  			 VARCHAR2(30);
  adreca  			 VARCHAR2(100);
  poblacio 			 VARCHAR2(50);
  codipostal		 VARCHAR2(5);
  telefon			 VARCHAR2(10);
  mail				 VARCHAR2(50);
  
  BEGIN
    lista:= VehiclesRevisioList();
	-- Obtenim els vehicles que van passar la ultima revisio fa 12 mesos
	query_string := 'SELECT VC.IDVEHICLECLIENT,VC.MATRICULA,VC.DATAULTIMAREVISIO,C.NOM,COGNOM1,C.COGNOM2,C.ADRECA,C.POBLACIO,C.CODIPOSTAL,C.TELEFON,C.MAIL
					FROM VEHICLECLIENT VC, CLIENT C
					WHERE VC.IDCLIENT = C.IDCLIENT AND TRUNC(MONTHS_BETWEEN(SYSDATE,VC.DATAULTIMAREVISIO)) = 12';
		
	OPEN cursorVehicles FOR query_string;
    LOOP
        FETCH cursorVehicles INTO idvehicleclient,matricula,dataultimarevisio,nom,cognom1,cognom2,adreca,poblacio,codipostal,telefon,mail;
        EXIT WHEN cursorVehicles%NOTFOUND;
		lista.extend;
		lista(lista.count) := vehicle_revisio_obj(idvehicleclient,matricula,dataultimarevisio,nom,cognom1,cognom2,adreca,poblacio,codipostal,telefon,mail);
    END LOOP;
    CLOSE cursorVehicles;

	RETURN lista;
	
	EXCEPTION  
        WHEN OTHERS THEN
          DBMS_OUTPUT.PUT_LINE('Error consultant vehicles pendents de revisio : (' || SQLCODE || ') ' || SQLERRM);     
END;
/