
/*****************************************************************
-- Nom:                 DW_CARREGARCLIENT
--
-- Descripci:          Carrega la taula de dimensio client
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARCLIENT" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_CLIENT (IDCLIENT, NIF, NOM, COGNOM1, COGNOM2, ADRECA, POBLACIO, CODIPOSTAL, TELEFON, MAIL)
		SELECT IDCLIENT, NIF, NOM, COGNOM1, COGNOM2, ADRECA, POBLACIO, CODIPOSTAL, TELEFON, MAIL FROM CLIENT 
		WHERE IDCLIENT NOT IN (SELECT IDCLIENT FROM DW_D_CLIENT);

	INSERIRLOG ('DW_CARREGARCLIENT','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW clients: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  INSERIRLOG ('DW_CARREGARCLIENT','',RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARCONCESSIONARI
--
-- Descripci:          Carrega la taula de dimensio concessionari
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARCONCESSIONARI" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_CONCESSIONARI (IDCONCESSIONARI, NOM, ADRECA, POBLACIO, CODIPOSTAL, TELEFON, MAIL)
		SELECT IDCONCESSIONARI, NOM, ADRECA, POBLACIO, CODIPOSTAL, TELEFON, MAIL FROM CONCESSIONARI
		WHERE IDCONCESSIONARI NOT IN (SELECT IDCONCESSIONARI FROM DW_D_CONCESSIONARI);
	
	INSERIRLOG ('DW_CARREGARCONCESSIONARI','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW concessionaris: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  INSERIRLOG ('DW_CARREGARCONCESSIONARI','',RESPOSTA);
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARCOMERCIAL
--
-- Descripci:          Carrega la taula de dimensio comercial
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARCOMERCIAL" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_COMERCIAL (IDCOMERCIAL, CONCESSIONARI, NUMEMPLEAT, NOM, COGNOM1, COGNOM2, TELEFON, MAIL)
		SELECT COM.IDCOMERCIAL, CON.NOM , COM.NUMEMPLEAT, COM.NOM, COM.COGNOM1, COM.COGNOM2, COM.TELEFON, COM.MAIL
		FROM COMERCIAL COM, CONCESSIONARI CON
		WHERE COM.IDCONCESSIONARI = CON.IDCONCESSIONARI
	    AND COM.IDCOMERCIAL NOT IN (SELECT IDCOMERCIAL FROM DW_D_COMERCIAL);
	
	INSERIRLOG ('DW_CARREGARCOMERCIAL','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW comercials: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
		  INSERIRLOG ('DW_CARREGARCOMERCIAL','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARCATALEGVEHICLE
--
-- Descripci:          Carrega la taula de dimensio cataleg vehicle
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARCATALEGVEHICLE" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_CATALEGVEHICLE (IDCONFIGURACIO, ANYCATALEG, MARCA, MODEL, COMBUSTIBLE, MOTOR, ACABAT)
		SELECT CV.IDCONFIGURACIO, CV.ANYCATALEG, M.DESCRIPCIO, MD.DESCRIPCIO, TC.DESCRIPCIO, MT.DESCRIPCIO, A.DESCRIPCIO
		FROM CATALEGVEHICLE CV, MARCA M, MODEL MD, TIPUSCOMBUSTIBLE TC, MOTOR MT, ACABAT A
		WHERE CV.IDMARCA = M.IDMARCA 
		AND CV.IDMODEL = MD.IDMODEL
		AND CV.IDCOMBUSTIBLE = TC.IDCOMBUSTIBLE
		AND CV.IDMOTOR = MT.IDMOTOR
		AND CV.IDACABAT = A.IDACABAT
		AND CV.IDCONFIGURACIO NOT IN (SELECT IDCONFIGURACIO FROM DW_D_CATALEGVEHICLE);
	
	INSERIRLOG ('DW_CARREGARCATALEGVEHICLE','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW cataleg vehicles: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARCATALEGVEHICLE','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARTALLER
--
-- Descripci:          Carrega la taula de dimensio taller
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARTALLER" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_TALLER (IDTALLER, CONCESSIONARI, ESPECIALITAT, NUMCITES, TELEFON, MAIL)
		SELECT TA.IDTALLER, CON.NOM , TA.ESPECIALITAT, TA.NUMCITES, TA.TELEFON, TA.MAIL
		FROM TALLER TA, CONCESSIONARI CON
		WHERE TA.IDCONCESSIONARI = CON.IDCONCESSIONARI
		AND TA.IDTALLER NOT IN (SELECT IDTALLER FROM DW_D_TALLER);
	
	INSERIRLOG ('DW_CARREGARTALLER','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW tallers: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARTALLER','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARVEHICLECLIENT
--
-- Descripci:          Carrega la taula de dimensio vehicles client
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARVEHICLECLIENT" (RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	INSERT INTO DW_D_VEHICLECLIENT (IDVEHICLECLIENT, BASTIDOR, CLIENT, MATRICULA, CONFIGURACIO)
		SELECT VC.IDVEHICLECLIENT, VC.BASTIDOR, C.NOM||' '||C.COGNOM1||' '||C.COGNOM2 AS CLIENT, VC.MATRICULA, M.DESCRIPCIO||' '||MD.DESCRIPCIO||' '||MT.DESCRIPCIO||' '||TC.DESCRIPCIO AS CONFIGURACIO
		FROM VEHICLECLIENT VC, CLIENT C, CATALEGVEHICLE CV, MARCA M, MODEL MD, MOTOR MT, TIPUSCOMBUSTIBLE TC
		WHERE VC.IDCLIENT = C.IDCLIENT
		AND VC.IDCONFIGURACIO = CV.IDCONFIGURACIO
		AND CV.IDMARCA = M.IDMARCA 
		AND CV.IDMODEL = MD.IDMODEL
		AND CV.IDCOMBUSTIBLE = TC.IDCOMBUSTIBLE
		AND CV.IDMOTOR = MT.IDMOTOR
		AND VC.IDVEHICLECLIENT NOT IN (SELECT IDVEHICLECLIENT FROM DW_D_VEHICLECLIENT);
	
	INSERIRLOG ('DW_CARREGARVEHICLECLIENT','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW vehicles client: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARVEHICLECLIENT','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARTEMPS
--
-- Descripci:          Carrega la taula de dimensio temps
--
-- Parmetres entrada:
						pDATAINICI DATE,
					    pDATAFI DATE
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARTEMPS" (
					   pDATAINICI DATE,
					   pDATAFI DATE,
					   RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  DATA_TMP DATE;
  
  BEGIN
    RESPOSTA:='OK';
	DATA_TMP:=pDATAINICI;

	WHILE DATA_TMP <= pDATAFI LOOP 
		--DBMS_OUTPUT.PUT_LINE('DIA--: ' || TO_CHAR(DATA_TMP,'YYYYMMDD') );
		INSERT INTO DW_D_TEMPS(IDTEMPS, DATA, DIA, MES, ANYO)
			SELECT TO_CHAR(DATA_TMP,'YYYYMMDD'), DATA_TMP, TO_CHAR(DATA_TMP,'DD'), TO_CHAR(DATA_TMP,'MM'), TO_CHAR(DATA_TMP,'YYYY')
			FROM DUAL
			WHERE TO_CHAR(DATA_TMP,'YYYYMMDD') NOT IN (SELECT IDTEMPS FROM DW_D_TEMPS);

			SELECT DATA_TMP+1 into DATA_TMP from dual;
	END LOOP;
	
	INSERIRLOG ('DW_CARREGARTEMPS','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW temps: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
		  INSERIRLOG ('DW_CARREGARTEMPS','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARVENDA
--
-- Descripci:          Carrega la taula de fets vendes
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARVENDA"  (RESPOSTA OUT NOCOPY VARCHAR2) 				   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	DELETE FROM DW_F_VENDA;

	INSERT INTO DW_F_VENDA (IDCOMERCIAL, IDCONCESSIONARI, IDCONFIGURACIO, IDCLIENT, IDTEMPS, IMPORT)
		SELECT V.IDCOMERCIAL, COM.IDCONCESSIONARI, P.IDCONFIGURACIO, P.IDCLIENT, TO_CHAR(V.DATAVENDA,'YYYYMMDD'), V.IMPORT
		FROM VENDA V, COMERCIAL COM, PRESSUPOST P
		WHERE V.IDCOMERCIAL = COM.IDCOMERCIAL
		AND V.IDPRESSUPOST = P.IDPRESSUPOST;
	
	INSERIRLOG ('DW_CARREGARVENDA','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW vendes: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARVENDA','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARREPARACIO
--
-- Descripci:          Carrega la taula de fets reparacions
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARREPARACIO"  (RESPOSTA OUT NOCOPY VARCHAR2) 				   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	DELETE FROM DW_F_REPARACIO;

	INSERT INTO DW_F_REPARACIO (IDVEHICLECLIENT, IDTALLER, IDTEMPS, TEMPSRESOLUCIO)
		SELECT A.IDVEHICLECLIENT, R.IDTALLER, TO_CHAR(R.DATAENTRADA,'YYYYMMDD'), R.TEMPSRESOLUCIO
		FROM REPARACIO R, AGENDA A 
		WHERE R.IDCITA = A.IDCITA;
		
	INSERIRLOG ('DW_CARREGARREPARACIO','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW reparacions: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARREPARACIO','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARREVISIO
--
-- Descripci:          Carrega la taula de fets revisions
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARREVISIO"  (RESPOSTA OUT NOCOPY VARCHAR2) 				   
  AS
  
  BEGIN
    RESPOSTA:='OK';

	DELETE FROM DW_F_REVISIO;

	INSERT INTO DW_F_REVISIO (IDVEHICLECLIENT, IDTALLER, IDTEMPS)
		SELECT A.IDVEHICLECLIENT, R.IDTALLER, TO_CHAR(R.DATAENTRADA,'YYYYMMDD')
		FROM REVISIO R, AGENDA A 
		WHERE R.IDCITA = A.IDCITA;
		
	INSERIRLOG ('DW_CARREGARREVISIO','',RESPOSTA);
		
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW revisio: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		  INSERIRLOG ('DW_CARREGARREVISIO','',RESPOSTA);		  
END;
/

/*****************************************************************
-- Nom:                 DW_CARREGARMAGATZEM
--
-- Descripci:          Carrega el magatzem al complet (fets i dimensi)
--
-- Parmetres entrada:
--
-- Error controlats:   
--
-- Autor: Ismael Guerrero
*****************************************************************/
CREATE OR REPLACE PROCEDURE "DW_CARREGARMAGATZEM"  (RESPOSTA OUT NOCOPY VARCHAR2) 				   
  AS
  DATAINICI DATE;
  DATAFI DATE;
  
  BEGIN
    RESPOSTA:='OK';

	DATAINICI := TO_DATE('01012010','DDMMYYYY');
	DATAFI := TO_DATE('31122012','DDMMYYYY');

	-- Carreguem les taules de dimesions
	DW_CARREGARCLIENT(RESPOSTA => RESPOSTA);
	DW_CARREGARCONCESSIONARI(RESPOSTA => RESPOSTA);
	DW_CARREGARCOMERCIAL(RESPOSTA => RESPOSTA);
	DW_CARREGARCATALEGVEHICLE(RESPOSTA => RESPOSTA);
	DW_CARREGARTALLER(RESPOSTA => RESPOSTA);
	DW_CARREGARVEHICLECLIENT(RESPOSTA => RESPOSTA);
	DW_CARREGARTEMPS(pDATAINICI => DATAINICI, pDATAFI => DATAFI, RESPOSTA => RESPOSTA);
	-- Carreguem les taules de fets
	DW_CARREGARVENDA(RESPOSTA => RESPOSTA);
	DW_CARREGARREPARACIO(RESPOSTA => RESPOSTA);
	DW_CARREGARREVISIO(RESPOSTA => RESPOSTA);
	
    EXCEPTION  
       WHEN OTHERS THEN
          RESPOSTA := 'Error carregant DW magatzem: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
END;
/